@@ -1,4 +1,6 @@
-import { parseStepsFromCells } from '@boyuai/jupyter-helper';
+import { parseCodeCell, parseStepsFromCells } from '@boyuai/jupyter-helper';
+import { parseQuestionOptions } from '@boyuai/jupyter-helper/lib/quiz';
+import { getQuizQuestionByUUID, parseQuizQuestions } from './services';
 import { ICell, IJupyter } from './types';
 
 const Jupyter: IJupyter = (window as any)['Jupyter'];
@@ -349,44 +351,35 @@ basepath = os.path.expanduser('~/share/bby/')
 #filepath = os.path.join(basepath, 'test.txt')`);
 };
 
-const syncLesson = function () {
+const syncLesson = async function () {
   const content = Jupyter.notebook.toJSON();
 
-  const _sync = () => {
-    const pathRegRes = workRegx.exec(Jupyter.notebook.notebook_path);
-    const baseRegRes = /user\/(dev-)?user-(\d+)/.exec(
-      Jupyter.notebook.base_url
-    );
-    if (!pathRegRes || !baseRegRes) {
-      alert('工作路径匹配失败');
-      return;
-    }
-    const lessonId = Number(pathRegRes[1]);
-    const token = pathRegRes[2];
-    const isDev = baseRegRes[1] === 'dev-';
-    const domain = isDev ? 'dev.boyuai.com' : 'www.boyuai.com';
+  const pathRegRes = workRegx.exec(Jupyter.notebook.notebook_path);
+  const baseRegRes = /user\/(dev-)?user-(\d+)/.exec(Jupyter.notebook.base_url);
+  if (!pathRegRes || !baseRegRes) {
+    alert('工作路径匹配失败');
+    return;
+  }
+  const isDev = baseRegRes[1] === 'dev-';
+  const domain = isDev ? 'dev.boyuai.com' : 'www.boyuai.com';
+  const lessonId = Number(pathRegRes[1]);
+  const token = pathRegRes[2];
+  const apiPrefix = `https://${domain}/api/v1`;
+  const userId = Number(baseRegRes[2]);
 
-    const userId = Number(baseRegRes[2]);
-    content.cells.forEach((c: ICell) => {
-      if (c.outputs) {
-        c.outputs = [];
-        c.execution_count = null;
-      }
-      // 清空非配置字段，避免影响用户数据
-      Object.keys(c.metadata).forEach((key) => {
-        if (!key.startsWith('bxy')) {
-          delete (c.metadata as any)[key];
-        }
-      });
-    });
+  // const lessonId = 1;
+  // const token = 'invalidtoken';
+  // const apiPrefix = 'http://localhost:3000';
+  // const userId = 5751;
+
+  const _sync = async () => {
     const body = {
       content,
       userId,
       token,
     };
-
-    const path = `/learn/admin/lessons/${lessonId}/jupyter`;
-    fetch(`https://${domain}/api/v1${path}`, {
+    const path = `${apiPrefix}/learn/admin/lessons/${lessonId}/jupyter`;
+    fetch(`${path}`, {
       body: JSON.stringify(body),
       headers: {
         'content-type': 'application/json',
@@ -409,15 +402,101 @@ const syncLesson = function () {
   };
 
   try {
+    // 校验 jupyter 格式
     parseStepsFromCells(content as any);
+
+    // 清理垃圾 & 冗余数据
+    for await (const c of content.cells as ICell[]) {
+      // 清理运行记录
+      if (c.outputs) {
+        c.outputs = [];
+        c.execution_count = null;
+      }
+
+      // 清空非配置字段，避免影响用户数据
+      Object.keys(c.metadata).forEach((key) => {
+        if (!key.startsWith('bxy')) {
+          delete (c.metadata as any)[key];
+        }
+      });
+
+      // 解析 Python Quiz Cell，未来会向 JSON 配置方向迁移，抛弃 Python 写法
+      const { type, code } = parseCodeCell(c.source);
+      if (type === 'quiz') {
+        const quizJSON = await parseQuizQuestions({
+          code,
+          userId,
+          lessonId,
+          token,
+          apiPrefix,
+        });
+        // console.log('quiz 解析结果', { quizJSON });
+
+        if (quizJSON?.statusCode) {
+          const quizError = new Error(`Quiz 解析失败: ${quizJSON?.message}`);
+          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
+          // @ts-ignore
+          quizError.detail = code;
+          throw quizError;
+        }
+
+        if (!quizJSON.value || !quizJSON.value.length) {
+          throw new Error('Quiz 解析失败: 缺少选项');
+        }
+
+        for await (const question of quizJSON.value) {
+          // 如果题目来自题库，获取当前最新数据
+          if (
+            typeof question.id === 'string' &&
+            question.id.startsWith('uuid-')
+          ) {
+            const uuid = question.id.slice(5);
+            const questionJSON = await getQuizQuestionByUUID({
+              uuid,
+              apiPrefix,
+            });
+            // console.log('来自题库的 quiz question', { questionJSON });
+            if (!questionJSON.uuid) {
+              throw new Error(`Quiz 解析失败: 题库题目 ${uuid} 解析异常`);
+            }
+            const questionIndex = quizJSON.value.findIndex(
+              (option: any) => option.id === `uuid-${uuid}`
+            );
+            if (questionIndex === -1) {
+              throw new Error(`Quiz 解析失败: 题库题目 ${uuid} 匹配失败`);
+            }
+            // console.log('开始更新 quiz quetion', {
+            //   questionWIP: quizJSON.value[questionIndex],
+            // });
+            const options = parseQuestionOptions(questionJSON);
+            quizJSON.value[questionIndex] = {
+              ...questionJSON,
+              id: questionJSON.uuid,
+              options,
+            };
+            console.log('quiz quetion 更新完成', {
+              question: quizJSON.value[questionIndex],
+            });
+          }
+
+          // 冗余 quiz 数据
+          if (!c.metadata.bxyQuizStep) {
+            c.metadata.bxyQuizStep = {};
+          }
+          c.metadata.bxyQuizStep.json = quizJSON;
+        }
+      }
+    }
+    // console.log({ cells: content.cells });
+
+    // 同步
     _sync();
   } catch (error) {
     Jupyter.dialog.modal({
-      title: '格式可能存在错误，请确认是否同步',
+      title: '可能存在错误，请确认是否同步',
       body:
-        `<p style="margin-bottom: 8px">Error: ${
-          error?.message || '未知错误'
-        }</p>` + `<pre>${JSON.stringify(error)}</pre>`,
+        `<p style="margin-bottom: 8px">${error?.message || '未知错误'}</p>` +
+        `<pre>${error?.detail || JSON.stringify(error)}</pre>`,
       sanitize: false,
       buttons: {
         确认同步: {
